﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using System.IO;

namespace MailMergeDocIO_Application.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult PerformMailmerge()
        {
            //Opens a Input Word Document.
            WordDocument wordDocument = new WordDocument(Server.MapPath("..//App_Data//Template.docx"), FormatType.Docx);
            #region Get DataTable
            //Data table to perform Mail merge.
            System.Data.DataTable dataTable = new System.Data.DataTable();
            //Data table name to identify the table field group.
            dataTable.TableName = "Datas";
            string[] fieldNames = new string[] { "Code", "Description", "Units", "Uom", "PayRate", "Factor", "Amount", "Start", "Finish" };
            foreach (string i in fieldNames)
                dataTable.Columns.Add(i);
            #endregion Get DataTable
            //Hide table if there is no data in data table
            if (dataTable.Rows.Count == 0)
            {
                //Gets Mail merge field collections to find fields in table
                MergeFieldCollectionExtenstion.GetMergeFieldCollections(wordDocument);
                //Removes field from collection.
                MergeFieldCollectionExtenstion.RemoveEmptyTable(dataTable.TableName, ref wordDocument);
            }
            //Perform mail merge other than empty data.
            wordDocument.MailMerge.ExecuteGroup(dataTable);

            #region Document Save
            MemoryStream memoryStream = new MemoryStream();
            wordDocument.Save(memoryStream, FormatType.Docx);
            memoryStream.Position = 0;
            //Setting HTTP content Type 
            string strContentType = "application/ms-word";
            //Display type
            string strDispType = "attachment";
            //Adding information to the header
            Response.AddHeader("Content-Type", strContentType);
            Response.AddHeader("Content-Disposition", string.Format("{0};filename={1};", strDispType, "Sample.docx"));
            memoryStream.CopyTo(Response.OutputStream);
            memoryStream.Dispose();
            Response.End();
            #endregion Document Save

            return View();
        }
        public ActionResult Index()
        {
            return View();
        }
    }
    public static class MergeFieldCollectionExtenstion
    {
        private static List<WMergeField> m_mergeFieldCollections = new List<WMergeField>();
        /// <summary>
        /// Gets Mail merge field collections from given WordDocument.
        /// </summary>
        /// <param name="wordDocument">Word Document</param>
        public static void GetMergeFieldCollections(WordDocument wordDocument)
        {
            //Iterate each sections from Word Document.
            for (int i = 0; i < wordDocument.Sections.Count; i++)
                FindMergeFieldInTextBodyCollection(wordDocument.Sections[i].Body.ChildEntities);
        }
        /// <summary>
        /// Removes empty Table from given WordDocument with table start group field.
        /// </summary>
        /// <param name="tableStartGroupName">Table start group field name</param>
        /// <param name="wordDocument">Word Document</param>
        public static void RemoveEmptyTable(string tableStartGroupName, ref WordDocument wordDocument)
        {
            if (m_mergeFieldCollections.Count != 0)
            {
                foreach (WMergeField field in m_mergeFieldCollections)
                {
                    if (field.FieldName == tableStartGroupName)
                    {
                        WTable table = GetOwnerTable(field) as WTable;
                        if (table != null)
                        {
                            for (int i = 0; i < wordDocument.Sections.Count; i++)
                            {
                                wordDocument.Sections[i].Body.ChildEntities.Remove(table);
                            }
                        }
                    }
                }
            }
        }
        #region Iterate MergeFields
        /// <summary>
        /// Finds the merge field in text body collection.
        /// </summary>
        /// <param name="entityCollection">The entity collection.</param>
        public static void FindMergeFieldInTextBodyCollection(EntityCollection entityCollection)
        {
            //Itreate the each text body collection to find the merge field.
            for (int i = 0; i < entityCollection.Count; i++)
            {
                if (entityCollection[i] is WParagraph)
                    FindMergeFieldInParagraph(entityCollection[i] as WParagraph);
                else if (entityCollection[i] is IWTable)
                    FindMergeFieldInTable(entityCollection[i] as IWTable);
            }
        }
        /// <summary>
        /// Finds the merge field in table.
        /// </summary>
        /// <param name="table">The table.</param>
        public static void FindMergeFieldInTable(IWTable table)
        {
            for (int i = 0; i < table.Rows.Count; i++)
            {
                for (int j = 0; j < table.Rows[i].Cells.Count; j++)
                {
                    WTableCell tableCell = table.Rows[i].Cells[j];
                    FindMergeFieldInTextBodyCollection(tableCell.ChildEntities);
                }
            }
        }
        /// <summary>
        /// Determines whether the merge field in paragraph or not.
        /// </summary>
        /// <param name="paragrph">The paragrph.</param>
        public static void FindMergeFieldInParagraph(WParagraph paragrph)
        {
            for (int i = 0; i < paragrph.ChildEntities.Count; i++)
            {
                if (paragrph.ChildEntities[i] is WMergeField)
                {
                    // Write the coding to process merge field.
                    if ((paragrph.ChildEntities[i] as WMergeField).Prefix == "TableStart")
                        m_mergeFieldCollections.Add(paragrph.ChildEntities[i] as WMergeField);
                }
                else if (paragrph.ChildEntities[i] is WTextBox)
                {
                    WTextBox textBox = paragrph.ChildEntities[i] as WTextBox;
                    FindMergeFieldInTextBodyCollection(textBox.ChildEntities);
                }
                else if (paragrph.ChildEntities[i] is Syncfusion.DocIO.DLS.Shape)
                {
                    Syncfusion.DocIO.DLS.Shape shape = paragrph.ChildEntities[i] as Syncfusion.DocIO.DLS.Shape;
                    FindMergeFieldInTextBodyCollection(shape.TextBody.ChildEntities);
                }
            }
        }
        /// <summary>
        /// Gets onwer table of paragraph.
        /// </summary>
        /// <param name="paragrph">The paragrph.</param>
        /// <returns>Owner table</returns>
        public static Entity GetOwnerTable(WMergeField field)
        {
            Entity baseEntity = field;
            do
            {
                if (baseEntity.Owner == null)
                    return baseEntity;
                baseEntity = baseEntity.Owner;
            }
            while (!(baseEntity is WTable));

            return baseEntity;
        }
        #endregion Iterate MergeFields
    }
}